#ifndef MS5611_H
#define MS5611_H

#include "mbed.h"

//_____ M A C R O S

#define MS5611_ADDR_W 0xEE // Module address write mode (CSBpin = 0);
#define MS5611_ADDR_R 0xEF // Module address read mode
#define MS5611_CMD_RESET 0x1E // ADC reset command
#define MS5611_CMD_ADC_READ 0x00 // ADC read command
#define MS5611_CMD_ADC_CONV 0x40 // ADC conversion command
#define MS5611_CMD_ADC_D1 0x00 // ADC D1 conversion
#define MS5611_CMD_ADC_D2 0x10 // ADC D2 conversion
#define MS5611_CMD_ADC_256 0x00 // ADC OSR=256
#define MS5611_CMD_ADC_512 0x02 // ADC OSR=512
#define MS5611_CMD_ADC_1024 0x04 // ADC OSR=1024
#define MS5611_CMD_ADC_2048 0x06 // ADC OSR=2048
#define MS5611_CMD_ADC_4096 0x08 // ADC OSR=4096
#define MS5611_CMD_PROM_RD 0xA0 // Prom read command

    /** Create ms5611 controller class
     *
     * @param ms5611 class
     *
     */
class ms5611 {

public:
    enum CSBpolarity {
        CSBpin_0,  //CSB pin is grounded, I2C address is 0xEE and 0xEF
        CSBpin_1,  //CSB pin is tied to Vdd, I2C address is 0xEC and 0xED
    };
    uint32_t C[8];
    /** Create a MS5611 object using the specified I2C object
     *   - User fixed I2C address 0xEE, CSB pin = 0
     *   - This is the default legacy constructor
     * @param sda - mbed I2C interface pin
     * @param scl - mbed I2C interface pin
     */
    ms5611(PinName sda, PinName scl);
    /** Create a MS5611 object using the specified I2C object
     *   - User defined use of the CSB pin
     *   - CSB pin = 0, user set I2C address to 0xEE
     *   - CSB pin = 1, user set I2C address to 0xEC 
     * @param sda - mbed I2C interface pin
     * @param scl - mbed I2C interface pin
     * @param ms5611::CSBpin_0 - CSB pin tied to ground
     * @param ms5611::CSBpin_1 - CSB pin tied to VDD
     */
    ms5611(PinName sda, PinName scl, CSBpolarity CSBpin);
    /** Initialize the MS5611 and set up the coefficients
     *    First - reset the MS5611
     *    Second - load coefficient values from the MS5611 PROM
     *    Third  - calculate coefficient checksum
     *  This routine only needs to be run once at boot up
     *
     * @param NONE
     */
    void cmd_reset();
    /** Calculate and return compensated temperature
     *    Returns double temperature in degC
     *
     * @param NONE
     */
    double calcTemp();
    /** Calculate and return compensated barometric pressure
     *    Returns double pressure in millibars
     *
     * @param NONE
     */
    double calcPressure();
    /** Return compensated barometric pressure
     *    Returns double pressure in millibars
     *    DOES NOT RE-CALCULATE FIRST!!!
     *    Saves time if you calcTemp(); first
     *
     * @param NONE
     */
    double getPressure();
    /** Calculate and returns altitude in feet
     *    Returns float altitude in feet
     *
     * @param float known pressure (mB) at sea level
     */
 
private:
#if not defined  MS5611i2cLOWLEVEL
    char cobuf[3];
#endif
    uint8_t _i2cWAddr;
    uint8_t _i2cRAddr;
    int m_i2c_start(bool readMode);
    void m_i2c_stop();
    uint8_t m_i2c_write(uint8_t data);
    uint8_t m_i2c_readAck();
    uint8_t m_i2c_readNak();
    void m_i2c_send(uint8_t cmd);
    void loadCoefs();
    uint64_t cmd_adc(uint8_t cmd);
    uint32_t cmd_prom(uint8_t coef_num);
    uint8_t crc4(uint32_t  n_prom[]);
    void calcPT();
    uint32_t PTbuffer[8];       // calibration coefficients
  
protected:
    I2C     _i2c;
    

}; 
#endif
